#include <algorithm>
#include <iostream>
#include <memory>

class IntArray {
	size_t size;
	int* array;

public:
	IntArray(size_t size)
	    : size(size)
	    , array(new int[size]) {}
	~IntArray() { delete[] array; }
	IntArray(const IntArray& other)
	    : size(other.size)
	    , array(new int[size]) {
		std::copy(other.array, other.array + size, array);
	}

	// variant no. 1; this one contains an error!
	IntArray& operator=(const IntArray& other) {
		delete[] array;
		size = other.size;
		array = new int[size];
		std::copy(other.array, other.array + size, array);
		return *this;
	}

	// variant no. 2, self-assignment detection
	/*IntArray& operator=(const IntArray& other) {
		if (&other == this) {
			return *this;
		}
		delete[] array;
		size  = other.size;
		array = new int[size];
		std::copy(other.array, other.array + size, array);
		return *this;
	}*/

	// variant no. 3, optimisation for same sizes
	/*IntArray& operator=(const IntArray& other) {
		if (&other == this) {
			return *this;
		}
		if (size != other.size) {
			delete[] array;
			size  = other.size;
			array = new int[size];
		}
		std::copy(other.array, other.array + size, array);
		return *this;
	}*/

	// variant no. 4, the copy-and-swap idiom
	/*void swap(IntArray& other) {
		using std::swap;
		swap(size, other.size);
		swap(array, other.array);
	}

	IntArray& operator=(IntArray other) {
		// other is a COPY (created by the copy constructor)
		swap(other); // we swap the insides of *this and other
		// other gets destroyed at the end of scope here
	}*/
};

int main() {
	IntArray arr(10);
	IntArray arr2 = arr;
	// ...
}
